/**
 * CS 3725 - Assignment #2
 * By: Tim Oram (200529220)
 * Timing Diagram
 *
 * Control Unit - One controller to rule them all
 */
package ca.mitmaro.mips;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.ListIterator;

import ca.mitmaro.mips.processor.ALU;
import ca.mitmaro.mips.processor.InstructionBuffer;
import ca.mitmaro.mips.processor.InstructionDecoder;

public class MIPS {
	
	private ALU[] alus;
	private InstructionDecoder id;
	private InstructionBuffer ib;
	
	private int add_cycle_time;
	private int multi_cycle_time;
	private int iu_cycle_time;
	
	private int cycle_counter;
	
	private LinkedList<Instruction> instructions;
	private LinkedList<Instruction> active_instructions;
	
	public MIPS(int num_alus, int add_cycles, int multi_cycles, int iu_cycles) {
		this.alus = new ALU[num_alus];
		this.add_cycle_time = add_cycles;
		this.multi_cycle_time = multi_cycles;
		this.iu_cycle_time = iu_cycles;
		this.reset();
	}
	
	public void reset() {

		int i;
		for (i = 0; i < this.alus.length; i++) {
			this.alus[i] = new ALU();
		}
		
		this.cycle_counter = 0;
		
		this.instructions = new LinkedList<Instruction>();
		this.active_instructions = new LinkedList<Instruction>();
		
		this.id = new InstructionDecoder(this.iu_cycle_time);
		this.ib = new InstructionBuffer();
		
	}
	
	public ALU getFreeAlu() {
		for (ALU alu: this.alus) {
			if (alu.isIdle()) {
				return alu;
			}
		}
		return null;
	}
	
	public void run(int[] instructions) {
		boolean end = false;
		
		int ii = 0;
		
		ALU alu;
		
		Instruction ins;
		
		while(true) {
			end = true;
			
			if (this.id.decoded() && this.ib.isEmpty()){
				end = false;
				this.ib.setInstruction(this.id.getInstruction());
				
			}
			
			if (this.id.ready() && ii < instructions.length) {
				end = false;
				this.id.setInstruction(instructions[ii], this.cycle_counter, ii);
				ii++;
			}
			
			// cycle
			for (ALU a: this.alus) {
				if (!a.isIdle()) {
					end = false;
				}
				a.cycle();
			}
			
			if ((alu = this.getFreeAlu()) != null && this.ib.hasInstruction()) {
				end = false;
				ins = this.ib.getInstruction();
				if (ins.getOperation() == Operation.ADD) {
					alu.set(this.add_cycle_time);
				} else if (ins.getOperation() == Operation.MULTI) {
					alu.set(this.multi_cycle_time);
				} else {
					//throw new RuntimeException("Invalid OP");
				}
				ins.setAlu(alu);
				ins.start();
				this.active_instructions.add(ins);
				this.instructions.add(ins);
			}
			
			ListIterator<Instruction> iter = this.active_instructions.listIterator();
			while(iter.hasNext()) {
				ins = iter.next();
				ins.cycle();
				
				// remove from active if not used
				if (ins.isDone()) {
					iter.remove();
				}
			}
			
			this.id.cycle();
			this.ib.cycle();
			
			if (!this.id.ready()) {
				end = false;
			}
			
			if (this.active_instructions.size() != 0) {
				end = false;
			}
			
			if (end) {
				break;
			}
			
			this.cycle_counter++;
			
		}
		
	}
	
	public void write() {
		DiagramWriter diawriter = new DiagramWriter(this.cycle_counter - 1);
		System.out.println("Timing Diagram of the Execution of a Program P in a Machine M\n");
		System.out.format(" Number of Instructions: %3d     Total Cycles: %3d     # D-Units: %3d\n", this.instructions.size(), this.cycle_counter - 1, this.alus.length);
		System.out.format("                 + time: %3d           * time: %3d      I-u time: %3d\n\n", this.add_cycle_time, this.multi_cycle_time, this.iu_cycle_time);
		diawriter.write(this.instructions);
		System.out.println();
	}
	
	public int[] loadInstructions(File file) throws IOException{
		
		BufferedInputStream in = new BufferedInputStream(new FileInputStream(file));
		
		byte[] d = new byte[4];
		
		ArrayList<Integer> instructions = new ArrayList<Integer>();
		
		int instruction = 0;
		
		// read while there were 4 bytes being read
		while(in.read(d, 0, 4) == 4) {
			instruction = 0;
			instruction |= (d[3] & 0xFF);
			instruction <<= 8;
			instruction |= (d[2] & 0xFF);
			instruction <<= 8;
			instruction |= (d[1] & 0xFF);
			instruction <<= 8;
			instruction |= (d[0] & 0xFF);
			instructions.add(instruction);
		}
		
		int[] rtn = new int[instructions.size()];
		
		int i = 0;
		for (Integer it: instructions) {
			rtn[i] = it.intValue();
			i++;
		}
		return rtn;
		
	}

	public static void main(String args[]) {
		
		int instructions[] = {24,32,32,24,32,24,24,32,24,24};
		
		MIPS mips = new MIPS(3,2,6,1);
		mips.run(instructions);
		mips.write();
		
	}
}
