/**
 * CS 3725 - Assignment #2
 * By: Tim Oram (200529220)
 * Timing Diagram
 * Instruction Buffer - Holds an instruction while waiting for the ALU to be free
 */
package ca.mitmaro.mips.processor;

import ca.mitmaro.mips.Instruction;

public class InstructionBuffer {
	
	private enum State {EMPTY, FULL};
	
	private Instruction instruction;
	
	private State state = State.EMPTY;
	
	public void setInstruction(Instruction ins) {
		if (this.state != State.EMPTY) {
			throw new InvalidStateException(String.format("Expecting EMPTY state. Buffer is in state %s", this.state));
		}
		this.state = State.FULL;
		this.instruction = ins;
	}
	
	public Instruction getInstruction() {
		if (this.state == State.EMPTY) {
			throw new InvalidStateException(String.format("Expecting FULL state. Buffer is in state %s", this.state));
		}
		this.state = State.EMPTY;
		return this.instruction;
	}
	
	public void cycle () {
		if (this.state == State.FULL) {
			this.instruction.cycle();
		}
	}
	
	public boolean isEmpty() {
		return this.state == State.EMPTY;
	}
	
	public boolean hasInstruction() {
		return this.state == State.FULL;
	}
}
