/**
 * CS 3725 - Assignment #2
 * By: Tim Oram (200529220)
 * Timing Diagram
 * Instruction Decoder - Instruction Decoding
 */
package ca.mitmaro.mips.processor;

import ca.mitmaro.mips.Instruction;
import ca.mitmaro.mips.Operation;

public class InstructionDecoder {
	
	private enum State {READY, OPERATING, DECODED};
	
	public Instruction instruction;
	
	private int required_cycles;
	
	private int current_cycle;
	
	private State state;
	
	public InstructionDecoder(int required_cycles) {
		this.required_cycles = required_cycles;
		this.current_cycle = 1;
		this.state = State.READY;
	}
	
	public void setInstruction(int instruction, int start_cycle, int instruction_number) {
		
		if (this.state != State.READY) {
			throw new InvalidStateException(String.format("Expecting READY state. Decoder at %s", this.state));
		}
		
		byte ins =  (byte)(instruction & 0xFFFF);
		
		if (ins == 0x20) {
			this.instruction = new Instruction(Operation.ADD);
		} else if (ins == 0x18) {
			this.instruction = new Instruction(Operation.MULTI);
		} else {
			throw new InvalidInstruction(String.format("Unknown instruction: %H", ins));
		}
		
		this.instruction.setStartCycle(start_cycle);
		this.instruction.setInstructionNumber(instruction_number);
		this.instruction.init(this.required_cycles);

		this.state = State.OPERATING;
	}

	// get the decoded instruction, once read the decoder will be reset
	public Instruction getInstruction() {
		if (this.state != State.DECODED) {
			throw new InvalidStateException(String.format("Expecting DECODED state. Decoder at %s", this.state));
		}
		this.state = State.READY;
		return this.instruction;
	}
	
	public void cycle() {
		
		switch (this.state) {
			case OPERATING:
				if (this.current_cycle == this.required_cycles) { // cycles complete
					this.current_cycle = 1;
					this.state = State.DECODED;
					return;
				}
				this.current_cycle++;
				break;
			case DECODED:
				this.instruction.cycle();
				break;
		}
		
		
	}
	
	// returns true once the decoder is ready for a new instruction
	public boolean ready() {
		return this.state == State.READY;
	}
	
	// returns true once the instruction has been decoded
	public boolean decoded() {
		return this.state == State.DECODED;
	}
	
}
