/**
 * CS 3725 - Assignment #2
 * By: Tim Oram (200529220)
 * Timing Diagram
 * Main Application
 */
package ca.mitmaro.mips;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

import ca.mitmaro.mips.MIPS;

// JCommander for parsing command line arguments
import com.beust.jcommander.JCommander;
import com.beust.jcommander.Parameter;
import com.beust.jcommander.ParameterException;

public class Application {

	@Parameter(names="-dunits", description="The number of D-Units.", required=true)
	private Integer dunits;
	@Parameter(names="-add-time", description="The number of cycles required for an add operation.", required=true)
	private Integer add_time;
	@Parameter(names="-multi-time", description="The number of cycles required for an multiplication operation.", required=true)
	private Integer multi_time;
	@Parameter(names="-iu-time", description="The number of cycles required for an instruction decode.", required=true)
	private Integer iu_time;
	@Parameter(names="-file", description="The file to read instructions.")
	private String file_path;
	
	private MIPS mips;
	
	public void interactiveMode() {
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		String line;
		int[] instructions;
		int index;
		
		System.out.println("Welcome to interactive mode. In this mode you will input a series of");
		System.out.println("'+' and '*' signs and the system will display the timing diagram of");
		System.out.println("that series of operations. Any invalid characters will be ignored. An");
		System.out.println("empty line will exit interactive mode.");
		System.out.println();
		System.out.println("Example Input: ++*+**+\n");
		
		try {
			System.out.print(">> ");
			while (!(line = in.readLine()).equals("")) {
				
				// remove invalid characters
				line = line.replaceAll("[^+*]", "");
				
				// no valid input, skip
				if (!line.equals("")) {
				
					instructions = new int[line.length()];
					index = 0;
					
					for (char c: line.toCharArray()) {
						if (c == '+') {
							instructions[index] = 0x20;
						} else {
							instructions[index] = 0x18;
						}
						index++;
					}
					
					this.mips.reset();
					this.mips.run(instructions);
					this.mips.write();
				} else {
					System.out.println("Not a valid program");
				}
				
				System.out.print(">> ");
			}
		} catch (IOException e){
			System.out.println("Unexpected IO Error. Exiting.");
			System.exit(1);
		}
	}
	
	public void run(String[] args) {
		
		JCommander jc = new JCommander();
		jc.setProgramName("MachineDiagram");
		
		try {
			jc.addObject(this);
			jc.parse(args);
		} catch (ParameterException e) {
			System.out.println(e.getMessage());
			System.out.println();
			System.out.println("Example Usage: MachineDiagram -dunits 3 -add-time 1 -iu-time 1 -multi-time 4");
			jc.usage();
			System.exit(1);
		}
		
		this.mips = new MIPS(this.dunits, this.add_time, this.multi_time, this.iu_time);
		
		// interactive mode
		if (this.file_path == null) {
			this.interactiveMode();
		} else { // file input
			File instruction_file = new File(this.file_path);
			if (!instruction_file.canRead()) {
				System.out.println(String.format("Unable to open file: %s", this.file_path));
				System.exit(1);
			}
			try {
				this.mips.run(this.mips.loadInstructions(instruction_file));
				this.mips.write();
			} catch (IOException e) {
				System.out.println(String.format("Error reading instruction file. Reason: %s", e.getMessage()));
				System.exit(1);
			}
		}
	}
}
