/**
 * CS 3725 - Assignment #2
 * By: Tim Oram (200529220)
 * Timing Diagram
 * Instruction
 */
package ca.mitmaro.mips;

import java.util.ArrayList;

import ca.mitmaro.mips.processor.ALU;

public class Instruction implements Comparable<Instruction> {
	
	public enum State {IDLE, READY, OPERATING, FINISHED}
	
	private int start_cycle;
	
	private Operation op;
	
	private ALU alu; // D-unit
	
	private int instruction_number; // index of instruction
	
	private ArrayList<Operation> operations;
	
	private State state;
	
	public Instruction(Operation op) {
		this.op = op;
		this.alu = null;
		this.operations = new ArrayList<Operation>();
		this.state = State.IDLE;
	}
	
	public void setStartCycle(int start_cycle) {
		this.start_cycle = start_cycle;
	}
	
	public void start() {
		this.state = State.READY;
	}
	
	public void init(int init_cycles) {
		// add init operations
		for (int i = 0; i < init_cycles; ++i) {
			this.operations.add(Operation.ID);
		}
	}
	
	public void cycle() {
		switch (this.state) {
			case OPERATING:
				if (this.alu.isOperating()) { // add operations while alu is working
					this.operations.add(this.op);
				} else { // alu is done
					this.state = State.FINISHED;
					this.operations.add(this.op);
				}
				break;
			case READY:
				this.state = State.OPERATING;
				break;
			default: // add no-ops for all other cycles 
				this.operations.add(Operation.NOP);
		}
	}
	
	public void setAlu(ALU alu) {
		this.alu = alu;
	}
	
	public boolean isDone() {
		return this.alu != null && this.alu.isIdle();
	}
	
	public int getStartCycle() {
		return this.start_cycle;
	}
	
	public Operation getOperation() {
		return this.op;
	}
	
	public ArrayList<Operation> getOpeartions() {
		return this.operations;
	}
	
	public void setInstructionNumber(int number) {
		this.instruction_number = number;
	}
	
	public int getInstructionNumber() {
		return this.instruction_number;
	}
	
	@Override
	public int compareTo(Instruction i) {
		return this.instruction_number - i.instruction_number;
	}
}