package ca.mitmaro.tomasulo;

import java.util.ArrayList;
import java.util.Map;

import ca.mitmaro.tomasulo.component.CommonDataBus;
import ca.mitmaro.tomasulo.component.RegisterTable;
import ca.mitmaro.tomasulo.component.ReservationStation;

public class Processor {
	
	ArrayList<ReservationStation> stations;

	RegisterTable register_table;
	RegisterTable execute_register_table;
	CommonDataBus common_data_bus;
	
	public Processor(RegisterTable register_table, RegisterTable exec_table, CommonDataBus common_data_bus) {
		this.register_table = register_table;
		this.execute_register_table = exec_table;
		this.common_data_bus = common_data_bus;
		this.stations = new ArrayList<ReservationStation>();
	}
	
	public void addStation(ReservationStation rs) {
		this.stations.add(rs);
	}
	
	public boolean dispatch(Instruction instruction) {
		
		Operation type = instruction.getType();
		
		ReservationStation rs = this.getFreeStation(type);
		
		// no station available, tell dispatch to stall
		if (rs == null) {
			return false;
		}
		
		rs.setBusy();

		// if register is not busy
		if (this.register_table.registerFree(instruction.getRegisterOne())) {
			rs.setValueOne();
		} else {
			// register was busy so set the tag for this value
			rs.setTagOne(this.register_table.getReservationStation(instruction.getRegisterOne()));
		}
		// and for the second operand
		if (this.register_table.registerFree(instruction.getRegisterTwo())) {
			rs.setValueTwo();
		} else {
			rs.setTagTwo(this.register_table.getReservationStation(instruction.getRegisterTwo()));
		}
		
		// tag register
		this.register_table.tagRegister(instruction.getResultRegister(), rs);
		
		return true;
	}
	
	public void issue() {
		
		Map.Entry<String, ReservationStation> data;
		
		for (ReservationStation rs: this.stations) {
			// skip free stations
			if (rs.isFree()) {
				continue;
			}
			
			if (rs.canExecute()) {
				// move to execute state
				this.execute_register_table.tagRegister(this.register_table.getRegister(rs) ,rs);
				// clear entry in registers
				this.register_table.clearRegister(rs);
			}
		}
		
		// get values from data bus and populate
		while ((data = this.common_data_bus.pop()) != null) {
			for (ReservationStation rs: this.stations) {
				rs.checkCopy(rs);
			}
		}
	}
	
	public void execute() {
		for (Map.Entry<String, ReservationStation> station: this.execute_register_table) {
			if (station.getValue() != null) {
				this.common_data_bus.put(station.getKey(), station.getValue());
				station.setValue(null);
			}
		}
	}
	
	private ReservationStation getFreeStation(Operation type) {
		for (ReservationStation rs: this.stations) {
			if (rs.isFree() && rs.getOperation() == type) {
				return rs;
			}
		}
		return null;
	}
}
