package ca.mitmaro.tomasulo.component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class RegisterTable implements Iterable<Map.Entry<String, ReservationStation>> {
	
	HashMap<String, ReservationStation> status = new HashMap<String, ReservationStation>();

	public RegisterTable(ArrayList<String> registers) {
		for (String reg: registers) {
			this.status.put(reg, null);
		}
	}

	public boolean registerFree(String register_name) {
		return this.status.get(register_name) == null;
	}
	
	public void tagRegister(String register_name, ReservationStation rs) {
		if (!this.registerFree(register_name)) {
			throw new AssertionError(String.format("The register, %s, is not free.", register_name));
		}
		this.status.put(register_name, rs);
	}
	
	public void clearRegister(ReservationStation rs) {
		for (Map.Entry<String, ReservationStation> station: this.status.entrySet()) {
			if (station.getValue().equals(rs)) {
				station.setValue(null);
			}
		}
	}
	
	public ReservationStation getReservationStation(String register_name) {
		return this.status.get(register_name);
	}

	public String getRegister(ReservationStation rs) {
		for (Map.Entry<String, ReservationStation> station: this.status.entrySet()) {
			if (station.getValue().equals(rs)) {
				return station.getKey();
			}
		}
		return null;
	}

	@Override
	public Iterator<Map.Entry<String, ReservationStation>> iterator() {
		return this.status.entrySet().iterator();
	}

}
