/**
 * CS3725 - Assignment #2
 * Tim Oram (200529220)
 * 
 * License:
 * The MIT License (MIT) Copyright (c) 2013 Tim Oram
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package ca.mitmaro.cs3725.a2.gui;

import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.text.BadLocationException;

import ca.mitmaro.cs3725.a2.Application;
import ca.mitmaro.io.FileUtil;

public class Events {

	private final Window window;
	private final Application application;
	private final GuiState state;

	private JFileChooser fileDialog;
	
	public Events(Window window, GuiState state, Application application) {
		this.window = window;
		this.application = application;
		this.state = state;
		
		this.fileDialog = new JFileChooser(System.getProperty("user.dir"));
		
		FileNameExtensionFilter filter = new FileNameExtensionFilter("ASM File", "asm");
		this.fileDialog.resetChoosableFileFilters();
		this.fileDialog.addChoosableFileFilter(filter);
		this.fileDialog.setFileFilter(filter);

	}
	
	public final AbstractAction programLoad = new AbstractAction() {

		private static final long serialVersionUID = 4985698366982066703L;
		
		private Events self = Events.this;

		@Override
		public void actionPerformed(ActionEvent e) {

			self.fileDialog.setVisible(true);
			if (self.fileDialog.showDialog(self.window, "Open") != JFileChooser.APPROVE_OPTION) {
				return;
			}
			
			try {
				BufferedReader in = (new FileUtil()).getBufferedReader(self.fileDialog.getSelectedFile());
				self.fileDialog.setSelectedFile(null);
				String line;
				
				self.window.editor.editor.setText("");
				while((line = in.readLine()) != null) {
					self.window.editor.editorDocument.insertString(
						self.window.editor.editorDocument.getLength(),
						line + "\n",
						null
					);
				}
				self.state.setEditorDirty(false);
			} catch (IOException e1) {
				e1.printStackTrace();
				return;
			} catch (BadLocationException e1) {
				throw new RuntimeException("Wrong index given to editor insert", e1);
			}
			
			
		}
	};

	public final AbstractAction programStart = new AbstractAction() {
		
		private static final long serialVersionUID = 291603021139806211L;
		
		private Events self = Events.this;
		
		@Override
		public void actionPerformed(ActionEvent e) {
			self.application.start();
		}
		
	};
	
	public final AbstractAction programSave = new AbstractAction() {

		private static final long serialVersionUID = 7452289284208899186L;
		
		private Events self = Events.this;
		
		@Override
		public void actionPerformed(ActionEvent e) {
			self.fileDialog.setVisible(true);
			if (self.fileDialog.showDialog(self.window, "Save") != JFileChooser.APPROVE_OPTION) {
				return;
			}
			
			try {
				BufferedWriter out = (new FileUtil()).getBufferedWriter(self.fileDialog.getSelectedFile());
				self.fileDialog.setSelectedFile(null);
				self.window.editor.editor.write(out);
				self.state.setEditorDirty(false);
			} catch (IOException e1) {
				e1.printStackTrace();
				return;
			}
		}
		
	};
	
	public final DocumentListener editorChanged = new DocumentListener() {

		private Events self = Events.this;
		
		@Override
		public void changedUpdate(DocumentEvent e) {
			self.state.setEditorDirty(true);
		}

		@Override
		public void insertUpdate(DocumentEvent e) {
			self.state.setEditorDirty(true);
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			self.state.setEditorDirty(true);
		}
		
	};
	
	public final AbstractAction editorClear = new AbstractAction() {

		private static final long serialVersionUID = 342346019612528049L;
		
		private Events self = Events.this;
		
		@Override
		public void actionPerformed(ActionEvent e) {
			self.window.editor.editor.setText("");
			self.state.updateGui();
		}
		
	};

	
	public final AbstractAction machineRun = new AbstractAction() {
		
		private static final long serialVersionUID = 7452289284208899186L;
		private Events self = Events.this;
		
		@Override
		public void actionPerformed(ActionEvent e) {
			self.application.createMachine(
				self.window.options.iuCyclesModel.getNumber().intValue(),
				self.window.options.registersModel.getNumber().intValue()
			);
			
			try {
				self.application.loadProgram(
					self.window.editor.editor.getDocument().getText(0, self.window.editor.editor.getDocument().getLength())
				);
			} catch (BadLocationException e1) {
				throw new RuntimeException(e1);
			} catch (IOException e1) {
				throw new RuntimeException(e1);
			}
			
			self.application.getMachine().clearDUnits();

			self.application.addDUnits(
				"+",
				self.window.options.adderCyclesModel.getNumber().intValue(),
				self.window.options.addersModel.getNumber().intValue()
			);

			self.application.addDUnits(
				"*",
				self.window.options.multiplierCyclesModel.getNumber().intValue(),
				self.window.options.multipliersModel.getNumber().intValue()
			);
			
			
		}
		
	};
	
	public final AbstractAction programPause = new AbstractAction() {
		

		private static final long serialVersionUID = -7931884768374950352L;
		
		private Events self = Events.this;
		
		@Override
		public void actionPerformed(ActionEvent e) {
			self.application.stop();
		}
		
	};

	public final AbstractAction programNext = new AbstractAction() {
		
		private static final long serialVersionUID = 1829970624807413708L;
		private Events self = Events.this;
		
		@Override
		public void actionPerformed(ActionEvent e) {
			self.application.next();
		}
		
	};
	
	public final AbstractAction programStop = new AbstractAction() {
		
		private static final long serialVersionUID = 5932638490492119153L;
		private Events self = Events.this;
		
		@Override
		public void actionPerformed(ActionEvent e) {
			self.application.reset();
		}
		
	};
	
	public final AbstractAction programAutoRun = new AbstractAction() {
		
		private static final long serialVersionUID = -1352426984225703600L;
		private Events self = Events.this;
		
		@Override
		public void actionPerformed(ActionEvent e) {
			self.application.autoRun(self.window.options.cycleTimeModel.getNumber().intValue());
		}
		
	};

}
