/**
 * CS3725 - Assignment #2
 * Tim Oram (200529220)
 * 
 * License:
 * The MIT License (MIT) Copyright (c) 2013 Tim Oram
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package ca.mitmaro.cs3725.a2.gui;

import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.table.DefaultTableCellRenderer;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;


public class Graph extends JPanel {

	private static final long serialVersionUID = 8366510659504389668L;

	public final JButton bAuto;
	public final JButton bPause;
	public final JButton bNext;
	public final JButton bStop;
	public final GraphModel graphModel;
	
	public Graph() {
		
		this.bAuto = new JButton("Auto Next");
		this.bNext = new JButton("Next");
		this.bPause = new JButton("Pause");
		this.bStop = new JButton("Stop");
		
		this.graphModel = new GraphModel();
		
		this.initComponents();
	}
	
	public void setupEvents(Events events) {
		this.bAuto.addActionListener(events.programAutoRun);
		this.bNext.addActionListener(events.programNext);
		this.bPause.addActionListener(events.programPause);
		this.bStop.addActionListener(events.programStop);
	}
	
	private void initComponents() {
		// this frame
		this.setBorder(BorderFactory.createTitledBorder("Timing Graph"));
		FormLayout layout = new FormLayout(
			"fill:pref:grow", "top:pref, fill:pref:grow"
		);
		this.setLayout(layout);
		//this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		CellConstraints cc = new CellConstraints();
		
		// buttons frame
		JPanel buttons = new JPanel();
		buttons.setLayout(new FlowLayout(FlowLayout.RIGHT, 0, 0));
		buttons.add(this.bAuto);
		buttons.add(this.bPause);
		buttons.add(this.bNext);
		buttons.add(this.bStop);
		this.add(buttons, cc.xy(1, 1));
		
		// graph
		final JTable table = new JTable(this.graphModel);
		table.setFillsViewportHeight(true);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		table.getColumnModel().getColumn(0).setPreferredWidth(35);
		table.getColumnModel().setColumnSelectionAllowed(false);
		((DefaultTableCellRenderer)table.getTableHeader().getDefaultRenderer())
			.setHorizontalAlignment(DefaultTableCellRenderer.CENTER);
		table.getColumnModel().addColumnModelListener(new TableColumnModelListener() {

			@Override
			public void columnAdded(TableColumnModelEvent e) {
				table.getColumnModel().getColumn(e.getToIndex()).setPreferredWidth(35);
			}
			public void columnMarginChanged(ChangeEvent e) {}
			public void columnMoved(TableColumnModelEvent e) {}
			public void columnRemoved(TableColumnModelEvent e) {}
			public void columnSelectionChanged(ListSelectionEvent e) {}
			
		});
		
		JScrollPane scroll = new JScrollPane(
			table,
			JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED
		);
		
		// graph
		this.add(scroll, cc.xy(1, 2));
	}
	
}
