/**
 * CS3725 - Assignment #2
 * Tim Oram (200529220)
 * 
 * License:
 * The MIT License (MIT) Copyright (c) 2013 Tim Oram
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package ca.mitmaro.cs3725.a2.machine.component;

import ca.mitmaro.cs3725.a2.machine.State;
import ca.mitmaro.cs3725.a2.program.Instruction;
import ca.mitmaro.cs3725.a2.program.Operation;

public class DUnit extends OperableComponent {
	
	private final RegisterBank registers;
	
	public DUnit(Operation operation, RegisterBank registers, int cycles) {
		super(operation, cycles);
		this.registers = registers;
	}
	
	
	@Override
	public State start(Instruction instruction) {
		
		// has a blocked register so conflict the dunit for now
		if (this.registers.isHeld(instruction)) {
			this.setInstruction(instruction);
			return this.setState(State.CONFLICT);
		}
		
		// registers are now being used
		this.registers.holdRegisters(instruction);
		
		return super.start(instruction);
	}
	
	// try to release a dunit from a conflicted state
	public void tryReleaseFromConflict() {
		if (this.getState() == State.CONFLICT) {
			if(!this.registers.isHeld(this.getInstruction())) {
				this.registers.unholdRegisters(this.getInstruction());
				this.setState(State.ACTIVE);
			}
		}
	}
	
	public boolean isConflict() {
		return this.getState() == State.CONFLICT;
	}


	@Override
	public State free() {
		this.registers.unholdRegisters(this.getInstruction());
		return super.free();
	}	
}
