/**
 * CS3725 - Assignment #2
 * Tim Oram (200529220)
 * 
 * License:
 * The MIT License (MIT) Copyright (c) 2013 Tim Oram
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package ca.mitmaro.cs3725.a2.machine.component;

import ca.mitmaro.cs3725.a2.machine.Cyclable;
import ca.mitmaro.cs3725.a2.machine.State;
import ca.mitmaro.cs3725.a2.program.Instruction;
import ca.mitmaro.cs3725.a2.program.Operation;

public abstract class OperableComponent implements Cyclable, Component {
	
	private State state;
	
	private int cycles;
	private int current_cycle;
	private Operation operation;
	private Instruction intruction;
	
	public OperableComponent(Operation operation, int cycles) {
		this.cycles = cycles;
		this.operation = operation;
		this.reset();
	}
	
	public State getState() {
		return this.state;
	}
	
	public State setState(State state) {
		State old = state;
		this.state = state;
		return old;
	}
	
	public State start(Instruction instruction) {
		
		if (this.state != State.FREE && this.state != State.INACTIVE) {
			// just a programmer check
			throw new RuntimeException("Trying to move from non-free state to working state");
		}
		
		this.intruction = instruction;
		
		return this.setState(State.ACTIVE);
	}
	
	public Operation getOperation() {
		return this.operation;
	}
	
	public void setInstruction(Instruction inst) {
		this.intruction = inst;
	}
	
	public Instruction getInstruction() {
		return this.intruction;
	}
	
	public State deactivate() {
		return this.setState(State.INACTIVE);
	}
	
	public State free() {
		return this.setState(State.FREE);
	}
	
	public boolean isFree() {
		return this.state == State.FREE || this.state == State.INACTIVE;
	}

	public boolean isActive() {
		return this.state == State.ACTIVE;
	}

	public boolean isInactive() {
		return this.state == State.INACTIVE;
	}
	
	public boolean isBlocked() {
		return this.state == State.BLOCKED;
	}
	
	public void cycle() {
		// set state from active to inactive when number of cycles have passed
		if (this.state == State.ACTIVE) {
			this.current_cycle++;
			if (this.cycles <= this.current_cycle) {
				this.setState(State.BLOCKED);
			}
		}
	}

	public boolean equals(Operation op) {
		return this.operation.equals(op);
	}
	
	public void reset() {
		this.current_cycle = 0;
		this.state = State.INACTIVE;
		this.intruction = null;
	}
}
