/**
 * CS3725 - Assignment #2
 * Tim Oram (200529220)
 * 
 * License:
 * The MIT License (MIT) Copyright (c) 2013 Tim Oram
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package ca.mitmaro.cs3725.a2.machine.component;

import java.util.HashMap;

import ca.mitmaro.cs3725.a2.program.Instruction;


public class RegisterBank implements Component {

	HashMap<Integer, Register> registers = new HashMap<Integer, Register>();
	
	public RegisterBank(int number_registers) {
		
		for (int i = 1; i <= number_registers; i++) {
			registers.put(i, new Register(i));
		}
		
	}
	
	public Register getRegister(int i) {
		return registers.get(i);
	}

	@Override
	public void reset() {
		for (Component c: this.registers.values()) {
			c.reset();
		}
	}
	
	public boolean isHeld(Instruction inst) {
		return (
			// IO
			this.registers.get(inst.getRS()).isHeldAsInput() ||
			// OI
			this.registers.get(inst.getRT()).isHeldAsOutput() ||
			this.registers.get(inst.getRD()).isHeldAsOutput() ||
			// OO
			this.registers.get(inst.getRS()).isHeldAsOutput()
		);
				
	}

	public void holdRegisters(Instruction inst) {
		// hold registers with their hold type (IO)
		this.registers.get(inst.getRS()).holdOutput();
		this.registers.get(inst.getRT()).holdInput();
		this.registers.get(inst.getRD()).holdInput();
	}

	public void unholdRegisters(Instruction inst) {
		// release registers from their hold type (IO)
		this.registers.get(inst.getRS()).releaseOutput();
		this.registers.get(inst.getRT()).releaseInput();
		this.registers.get(inst.getRD()).releaseInput();
	}
}
