/**
 * CS3725 - Assignment #3
 * Tim Oram (200529220)
 * 
 * License:
 * The MIT License (MIT) Copyright (c) 2013 Tim Oram
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package ca.mitmaro.cs3725.a3.gui.model;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

import ca.mitmaro.cs3725.a3.program.Operation;


public final class GraphModel extends AbstractTableModel {

	private ArrayList<ArrayList<Operation>> cycles;
	
	private int rowCount;
	
	private static final long serialVersionUID = 3483898013886457014L;
	
	public GraphModel() {
		this.reset();
	}

	public void addCycle(ArrayList<Operation> cycle) {
		this.cycles.add(cycle);
		int rows = cycle.size();
		
		if (rows > this.rowCount) {
			this.rowCount = rows;
		}
		this.fireTableStructureChanged();
	}
	
	public void reset() {
		this.rowCount = 0;
		this.cycles = new ArrayList<ArrayList<Operation>>();
	}
	
	public String getColumnName(int column) {
		if (column == 0) {
			return "I/C";
		}
		
		return String.format("C%s", column);
	}
	
	@Override
	public int getColumnCount() {
		return this.cycles.size() + 1;
	}

	@Override
	public int getRowCount() {
		return this.rowCount;
	}

	@Override
	public boolean isCellEditable(int row, int col) {
		return false;
	}
	
	@Override
	public Object getValueAt(int row, int col) {
		
		if (col == 0) {
			return String.format("I%d", row);
		}
		
		// does not have col
		if (col > this.cycles.size()) {
			return "";
		}
		
		// does not have row
		if (row >= this.cycles.get(col-1).size()) {
			return "";
		}
		
		return this.cycles.get(col-1).get(row).toString();
	}
	
}