/**
 * CS3725 - Assignment #3
 * Tim Oram (200529220)
 * 
 * License:
 * The MIT License (MIT) Copyright (c) 2013 Tim Oram
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package ca.mitmaro.cs3725.a3.gui.model;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ca.mitmaro.cs3725.a3.machine.component.Register;


public final class RegisterModel extends AbstractTableModel {

	private static Logger logger = LoggerFactory.getLogger(RegisterModel.class);
	
	private static final long serialVersionUID = 4817248212829996037L;
	
	private ArrayList<Register> registers;
	
	public RegisterModel() {
		this.registers = new ArrayList<Register>();
	}

	public void setRegisters(ArrayList<Register> registers) {
		this.registers = registers;
		logger.info("Loading {} registers", registers.size());
		this.fireTableStructureChanged();
	}
	
	public void setRegister(int reg, Register register) {
		this.registers.set(reg-1, register);
		this.fireTableStructureChanged();
	}
	
	@Override
	public String getColumnName(int column) {
		
		switch (column) {
		case 0: return "Register";
		case 1: return "Status";
		}
		
		return "ERROR";
	}
	
	@Override
	public int getColumnCount() {
		return 2;
	}

	@Override
	public int getRowCount() {
		return this.registers.size();
	}

	@Override
	public boolean isCellEditable(int row, int col) {
		return false;
	}
	
	@Override
	public Object getValueAt(int row, int col) {
		switch(col) {
		case 0: return this.registers.get(row).getNumber();
		case 1: return (this.registers.get(row).isFree() ? "Free" : "Busy");
		}
		
		return "-";
		
	}
	
}