/**
 * CS3725 - Assignment #3
 * Tim Oram (200529220)
 * 
 * License:
 * The MIT License (MIT) Copyright (c) 2013 Tim Oram
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package ca.mitmaro.cs3725.a3.machine.component;

import ca.mitmaro.cs3725.a3.machine.Cyclable;
import ca.mitmaro.cs3725.a3.machine.State;
import ca.mitmaro.cs3725.a3.program.Instruction;
import ca.mitmaro.cs3725.a3.program.Operation;

public abstract class OperableComponent implements Cyclable {
	
	public State state;
	
	private int cycles;
	private int current_cycle;
	public final Operation operation;
	private Instruction intruction;
	
	public OperableComponent(Operation operation, int cycles) {
		this.cycles = cycles;
		this.operation = operation;
		this.reset();
	}
	
	public void start(Instruction instruction) {
		this.intruction = instruction;
		this.state = State.READY;
	}
	
	public void reset() {
		this.current_cycle = 0;
		this.state = State.FREE;
		//this.intruction = null;
	}
	
	@Override
	public void cycle() {
		
		
		switch (this.state) {
			case BUSY:
				this.current_cycle++;
				
				assert this.current_cycle > this.cycles: this.current_cycle;
				
				if (this.current_cycle == this.cycles) {
					this.state = State.DONE;
				}
				break;
			case READY:
				this.state = State.BUSY;
				break;
		default:
			break;
				
		}
	}
	
	public Instruction getInstruction() {
		return this.intruction;
	}
	
	public Operation getOperation() {
		return this.operation;
	}
	
	public boolean isFree() {
		return this.state == State.FREE;
	}
	
	public boolean isReady() {
		return this.state == State.READY;
	}
	
	public boolean isDone() {
		return this.state == State.DONE;
	}
	
	public boolean isBusy() {
		return this.state == State.BUSY;
	}
	
	public boolean isConflicted() {
		return this.state == State.CONFLICT;
	}
	
	public boolean equals(Operation op) {
		return this.operation.equals(op);
	}
	
	
}
