/**
 * CS3725 - Assignment #3
 * Tim Oram (200529220)
 * 
 * License:
 * The MIT License (MIT) Copyright (c) 2013 Tim Oram
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package ca.mitmaro.cs3725.a3.program;

// takes a string and created an instruction object
public class Instruction {
	
	private String line;
	
	private Operation operation;
	
	private boolean error;
	
	// register numbers
	private int rs = -1;
	private int rt = -1;
	private int rd = -1;
	
	// invalid stuff (wasn't used after but may use it in next assignment)
	private String invalid_message = null;
	
	// memory location for this instruction
	private int pc;
	
	public Instruction(Instruction inst) {
		this.rs = inst.rs;
		this.pc = inst.pc;
		this.line = inst.line;
		this.error = inst.error;
	}
	
	public Instruction(int pc, String line) {
		this.line = line;
		this.pc = pc;
		this.error = false;

		String[] tmp = line.split("\\s+", 4);
		
		// check valid instruction
		if (tmp.length != 4) {
			this.setInvalid("Syntax Error: Invalid instruction format");
			return;
		}
		
		// parse function
		if (tmp[0].equals("add")) {
			this.operation = new Operation("+");
		} else if (tmp[0].equals("mul")) {
			this.operation = new Operation("*");
		} else {
			this.setInvalid("Syntax Error: Invalid operation provided");
			return;
		}
		
		// set registers
		
		// rs
		try {
			this.rs = Integer.parseInt(tmp[2]);
		} catch (NumberFormatException e) {
			this.rs = -1;
		}
		if (this.rs == -1) {
			this.setInvalid("Syntax Error: Invalid register number provided for RS");
		}
		
		// rt
		try {
			this.rt = Integer.parseInt(tmp[3]);
		} catch (NumberFormatException e) {
			this.rt = -1;
		}
		if (this.rt == -1) {
			this.setInvalid("Syntax Error: Invalid register number provided for RT");
		}
		
		// rd
		try {
			this.rd = Integer.parseInt(tmp[1]);
		} catch (NumberFormatException e) {
			this.rd = -1;
		}
		if (this.rd == -1) {
			this.setInvalid("Syntax Error: Invalid register number provided for RD");
		}
		
	}

	public String invalidMessage() {
		return this.invalid_message;
	}
	
	public Operation getOperation() {
		return this.operation;
	}

	public int getRS() {
		return this.rs;
	}
	
	public int getRT() {
		return this.rt;
	}
	
	public int getRD() {
		return this.rd;
	}
	
	public int getPC() {
		return this.pc;
	}
	
	public String getLine() {
		return this.line;
	}
	
	private void setInvalid(String message) {
		this.error = true;
		this.invalid_message = message;
	}

}
